/*********************************************************************
  All ADuC7023 library code provided by ADI, including this file, is provided
 as is without warranty of any kind, either expressed or implied.
 You assume any and all risk from the use of this code.  It is the
 responsibility of the person integrating this code into an application
 to ensure that the resulting application performs as required and is safe.

 Author        : ADI - Apps            www.analog.com/ADuC7023

 Date          : February 2009

 File          : ADC.c

 Hardware      : Applicable to ADuC7023 rev A later silicon
                 
 Description   : Program that samples the ADC and outputs them to I2C0 channel.
 				I2C0 is an I2C Slave device.
		
*********************************************************************/
// Bit Definitions
#define BIT0  0x01
#define BIT1  0x02
#define BIT2  0x04
#define BIT3  0x08
#define BIT4  0x10
#define BIT5  0x20
#define BIT6  0x40
#define BIT7  0x80
#define BIT8  0x100
#define BIT9  0x200
#define BIT10 0x400
#define BIT11 0x800
#define BIT12 0x1000
#define BIT13 0x2000
#define BIT14 0x4000
#define BIT15 0x8000
#include <aduc7023.h>

unsigned short ADCDATA[256];
int n,i,x = 0;
unsigned char ucReadytoRead = 0;
unsigned char ucI2CData, uc2Bytes = 0;
void ADCpoweron(int);
int main(void) 
{
	POWKEY1 = 0x01;							// Configure CPU Clock for 41.78MHz, CD=0
 	POWCON = 0x00;
 	POWKEY2 = 0xF4;	
	GP0DAT |= 0x80000000;
	REFCON = 0x01;							// internal 2.5V reference. 2.5V on Vref pin
	ADCpoweron(20000);						// power on ADC
	ADCCP = 0x00;	  						// select ADC channel 0
	ADCCN = 0x01;	  						// select ADC channel 1

	POWKEY3 = 0x76;			 				// Enable PWM, SPI, I2C0, I2C1 Peripherals with 41.78MHz clock
    POWCON1 = 0x924;					
	POWKEY4 = 0xB1;
	// Configure P0.6 and P0.7 for I2C mode
	GP0CON |= 0x0110000;					// Only write to bits 4/5
	I2C0SCON = BIT0 		  				// Enable I2C Slave + Enable Rx interrupt +
			+ BIT9 + BIT10  				// Enable Rx interrupt + Enable Tx interrupt +
			+ BIT8;							// Enable "Stop-after-Start-Detected" IRQ.
	I2C0ID0  = 0xA0;						// Configure Slave's I2C network address
	I2C0FSTA = BIT8;						// Flush Slave Tx FIFO
	I2C0FSTA &= ~BIT8;
	
	ADCCON = 0xEEC;							// ADC Config: fADC/8, acq. time = 8 clocks, Differential mode, Continuous conversion mode
    IRQEN = BIT6 + BIT9;					// Enable ADC and I2C1 Slave interrupt sources
	while (1) 
	{
	   if (ucReadytoRead == 1)
	   {
	   	  I2C0FSTA = BIT8;					// Flush Slave Tx FIFO
		  I2C0FSTA &= ~BIT8;
		  ucI2CData = (unsigned char)(ADCDATA[0]>>8);
		  I2C0STX = ucI2CData;
		  GP0DAT ^= 0x800000;
		  ucI2CData = (unsigned char)(ADCDATA[0]);
		  n = 1;
		  uc2Bytes =0;		
		  ucReadytoRead = 0;
	   }	   	 
	}				
}
void IRQ_Handler(void) __irq
{
	unsigned long IRQSTATUS = 0;
	unsigned int I2CSSTATUS = 0;

	IRQSTATUS = IRQSTA;	   					// Read off IRQSTA register
	if ((IRQSTATUS & BIT6) == BIT6)			// ADC interrupt source
	{
		ADCDATA[i++] = (ADCDAT >> 16);		// Read ADC result
		if (i == 256)
		{
		    i = 0;
			ucReadytoRead = 0x1;
		}
	}
	if ((IRQSTATUS & BIT9) == BIT9)			// I2C1 Slave interrupt source
	{
		I2CSSTATUS = I2C0SSTA;		 		//	Read I2C1 status register
		if ((I2CSSTATUS & BIT3) == BIT3) 	// If I2C Slave Rx IRQ
	   	{

	   	}
	   if ((I2CSSTATUS & BIT2) == BIT2) 	// If I2C Master Tx IRQ
	   	{
			I2C0STX = ucI2CData;
			GP0DAT ^= 0x800000;
			uc2Bytes++;
			if (uc2Bytes == 2)
			{
			    ucI2CData = (ADCDATA[n]);
				uc2Bytes = 0;
				n++;
			}
			else
			{
				ucI2CData = (unsigned char)(ADCDATA[n]>>8);
			}
      	}   
	
	   if ((I2CSSTATUS & BIT10) == BIT10) 	// If I2C Stop after Start condition detected IRQ
	   	{
			n=0;
	   	}
	   if ((I2CSSTATUS & BIT13) == BIT13) 	// If I2C Repeated Start detected IRQ
	  	{

	    }
		if ((I2CSSTATUS & BIT14) == BIT14) 	//  Start detected IRQ
	  	{
		   i = 0;
		   IRQCLR = BIT6;					// Disable ADC interrupts while communicating over I2C
	    }
	}
}
void ADCpoweron(int time)
{
	ADCCON = 0x20;	 						// power-on the ADC
	while (time >=0)	  					// wait for ADC to be fully powered on
    time--;
}
